@@ -7,10 +7,14 @@
 from talos.thrift.auth.ttypes import Credential
 from talos.thrift.auth import constants
 from talos.thrift.transport.TTransport import TTransportBase
+from talos.client.TalosErrors import TalosTransportError
+from talos.client.TalosErrors import HttpStatusCode
 from io import BytesIO
 from datetime import datetime
 from six.moves import urllib
 from six.moves import http_client
+from talos.utils.Utils import current_time_mills
+import re
 import six
 import time
 import hashlib
@@ -21,6 +25,8 @@ import ssl
 import sys
 import uuid
 import logging
+import dns.resolver
+import random
 import traceback
 
 
@@ -104,10 +110,8 @@ class TalosHttpClient(TTransportBase):
         NOTE: if both cafile and ssl_context are defined, ssl_context will override cafile.
         """
         if port is not None:
-            self.logger.warning(
-                "Please use the THttpClient('http{s}://host:port/path') constructor",
-                DeprecationWarning,
-                stacklevel=2)
+            self.logger.warning("Please use the THttpClient('http{s}://host:port/path') "
+                                + "constructor", DeprecationWarning, stacklevel=2)
             self.host = uri_or_host
             self.port = port
             assert path
@@ -314,16 +318,16 @@ class TalosHttpClient(TTransportBase):
 
     def open(self):
         if self.scheme == 'http':
-            self.__http = http_client.HTTPConnection(self.host, self.port,
-                                                     timeout=self.__timeout)
+            self.__http = http_client.HTTPConnection(self.get_addr_by_host(self.host),
+                                                     self.port, timeout=self.__timeout)
         elif self.scheme == 'https':
-            self.__http = http_client.HTTPSConnection(self.host, self.port,
-                                                      key_file=self.keyfile,
+            self.__http = http_client.HTTPSConnection(self.get_addr_by_host(self.host),
+                                                      self.port, key_file=self.keyfile,
                                                       cert_file=self.certfile,
                                                       timeout=self.__timeout,
                                                       context=self.context)
         if self.using_proxy():
-            self.__http.set_tunnel(self.realhost, self.realport,
+            self.__http.set_tunnel(self.get_addr_by_host(self.realhost), self.realport,
                                    {"Proxy-Authorization": self.proxy_auth})
 
     def close(self):
@@ -395,15 +399,43 @@ class TalosHttpClient(TTransportBase):
 
         # Get reply to flush the request
         self.__http_response = self.__http.getresponse()
-        response = self.__http_response
         self.code = self.__http_response.status
-        self.message = self.__http_response.reason
-        self._headers = self.__http_response.msg
+        response = self.__http_response
         if not self.code == 200:
-            pass
-        # TODO Trans Exception to TalosTransportError
-        # reason = response.fp.read()
-        # raise TalosTransportError(self.code, self.message, current_time_mills())
+            isTalosTransportError = True
+            headers = response.headers._headers
+            for i in headers:
+                if i[0] == 'Content-Type':
+                    isTalosTransportError = False
+                    if not re.match('application/x-thrift', i[1]):
+                        isTalosTransportError = True
+
+            if isTalosTransportError:
+                self.adjustClock(response, self.code)
+                raise TalosTransportError(self.code, response.reason,
+                                          current_time_mills())
+
+    # Adjust local clock when clock skew error received from server.The client clock need to be
+    # roughly synchronized with server clock to make signature secure
+    # and reduce the chance of replay attacks.
+    # @ param response server response
+    # @ param httpStatusCode status code
+    # @ return if clock is adjusted
+    def adjustClock(self, response, httpStatusCode):
+        if httpStatusCode == HttpStatusCode.CLOCK_TOO_SKEWED:
+            headers = response.heasers(constants.HK_TIMESTAMP)
+            for key, hv in headers.items():
+                serverTime = int(hv)
+                min = 60 * 60 * 24 * 365 * (2010 - 1970)
+                max = 60 * 60 * 24 * 365 * (2030 - 1970)
+            if serverTime > min and serverTime < max:
+                self.logger.debug("Adjusting client time from {} to {}" +
+                                  str(self._clockOffset) + " serverTime: " + str(
+                    serverTime))
+                self._clockOffset = serverTime
+                return True
+        return False
+
 
     def generate_random_id(self, length=None):
         return str(uuid.uuid4())[0:length]
@@ -417,3 +449,22 @@ class TalosHttpClient(TTransportBase):
     def set_sid(self, sid=None):
         self._sid = sid
 
+    # for dns cache
+    def get_addr_by_host(self, host=None):
+        ipList = []
+        try:
+            A = dns.resolver.query(host)
+            if len(A.response.answer) > 0:
+                for items in A.response.answer:
+                    for item in items:
+                        ipList.append(item)
+                return str(random.choice(ipList))
+        except Exception as e:
+            self.logger.info("dns resolve error for host: "
+                             + host + str(traceback.format_exc()))
+        return host
+
+
+
+
+
